@@ -19,6 +19,7 @@
 #include <xtensor/xshape.hpp>
 #include <xtensor/xshape.hpp>
 #include "xtensor/xstrided_view.hpp"
+#include "xtensor/xchunked_view.hpp"
 
 namespace xt
 {
@@ -31,35 +32,10 @@ namespace xt
     template <class CTE>
     class zexpression_wrapper;
 
-    namespace detail
-    {
-        template <class E>
-        struct is_xstrided_view : std::false_type
-        {
-        };
-
-        template <class CT, class S, layout_type L, class FST>
-        struct is_xstrided_view<xstrided_view<CT, S, L, FST>> : std::true_type
-        {
-        };
-
-        template <class CT>
-        using is_const = std::is_const<std::remove_reference_t<CT>>;
-
-        template <class CT, class R = void>
-        using disable_xstrided_view_t = std::enable_if_t<!is_xstrided_view<CT>::value || is_const<CT>::value, R>;
-
-        template <class CT, class R = void>
-        using enable_const_t = std::enable_if_t<is_const<CT>::value, R>;
-
-        template <class CT, class R = void>
-        using disable_const_t = std::enable_if_t<!is_const<CT>::value, R>;
-    }
-
     /******************
      * zarray builder *
      ******************/
-
+    
     namespace detail
     {
         template <class E>
@@ -82,6 +58,16 @@ namespace xt
         {
         };
 
+        template <class E>
+        struct is_xstrided_view : std::false_type
+        {
+        };
+
+        template <class CT, class S, layout_type L, class FST>
+        struct is_xstrided_view<xstrided_view<CT, S, L, FST>> : std::true_type
+        {
+        };
+
         template <class E>
         struct zwrapper_builder
         {
@@ -106,85 +92,18 @@ namespace xt
         {
             return zwrapper_builder<E>::run(std::forward<E>(e));
         }
-    }
 
-    const std::string endianness_string = (xtl::endianness() == xtl::endian::little_endian) ? "<" : ">";
+        template <class CT>
+        using is_const = std::is_const<std::remove_reference_t<CT>>;
 
-    template <class T>
-    inline void set_data_type(nlohmann::json& metadata)
-    {
-    }
+        template <class CT, class R = void>
+        using disable_xstrided_view_t = std::enable_if_t<!is_xstrided_view<CT>::value || is_const<CT>::value, R>;
 
-    template <>
-    inline void set_data_type<bool>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = "bool";
-    }
+        template <class CT, class R = void>
+        using enable_const_t = std::enable_if_t<is_const<CT>::value, R>;
 
-    template <>
-    inline void set_data_type<uint8_t>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = "u1";
-    }
-
-    template <>
-    inline void set_data_type<int8_t>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = "i1";
-    }
-
-    template <>
-    inline void set_data_type<int16_t>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = endianness_string + "i2";
-    }
-
-    template <>
-    inline void set_data_type<uint16_t>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = endianness_string + "u2";
-    }
-
-    template <>
-    inline void set_data_type<int32_t>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = endianness_string + "i4";
-    }
-
-    template <>
-    inline void set_data_type<uint32_t>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = endianness_string + "u4";
-    }
-
-    template <>
-    inline void set_data_type<int64_t>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = endianness_string + "i8";
-    }
-
-    template <>
-    inline void set_data_type<uint64_t>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = endianness_string + "u8";
-    }
-
-    template <>
-    inline void set_data_type<xtl::half_float>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = endianness_string + "f2";
-    }
-
-    template <>
-    inline void set_data_type<float>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = endianness_string + "f4";
-    }
-
-    template <>
-    inline void set_data_type<double>(nlohmann::json& metadata)
-    {
-        metadata["data_type"] = endianness_string + "f8";
+        template <class CT, class R = void>
+        using disable_const_t = std::enable_if_t<!is_const<CT>::value, R>;
     }
 
     /*************************
@@ -233,10 +152,14 @@ namespace xt
 
         virtual self_type* clone() const = 0;
 
+        virtual bool is_array() const = 0;
+        virtual bool is_chunked() const = 0;
+
         virtual self_type* strided_view(xstrided_slice_vector& slices) = 0;
 
         virtual const nlohmann::json& get_metadata() const = 0;
         virtual void set_metadata(const nlohmann::json& metadata) = 0;
+
         virtual std::size_t dimension() const = 0;
         virtual const shape_type& shape() const = 0;
         virtual void resize(const shape_type& shape) = 0;
@@ -262,13 +185,13 @@ namespace xt
 
         using base_type = zarray_impl;
         using shape_type = base_type::shape_type;
+        using slice_vector = xstrided_slice_vector;
 
         virtual ~ztyped_array() = default;
 
-        virtual bool is_array() const = 0;
-
         virtual xarray<T>& get_array() = 0;
         virtual const xarray<T>& get_array() const = 0;
+        virtual xarray<T> get_chunk(const slice_vector& slices) const = 0;
 
         virtual void assign(xarray<T>&& rhs) = 0;
 
@@ -293,6 +216,7 @@ namespace xt
         using value_type = typename std::decay_t<CTE>::value_type;
         using base_type = ztyped_array<value_type>;
         using shape_type = typename base_type::shape_type;
+        using slice_vector = typename base_type::slice_vector;
 
         template <class E>
         zexpression_wrapper(E&& e);
@@ -300,15 +224,17 @@ namespace xt
         virtual ~zexpression_wrapper() = default;
 
         bool is_array() const override;
+        bool is_chunked() const override;
 
         xarray<value_type>& get_array() override;
         const xarray<value_type>& get_array() const override;
+        xarray<value_type> get_chunk(const slice_vector& slices) const override;
 
         void assign(xarray<value_type>&& rhs) override;
 
         self_type* clone() const override;
 
-        zarray_impl* strided_view(xstrided_slice_vector& slices) override;
+        zarray_impl* strided_view(slice_vector& slices) override;
 
         const nlohmann::json& get_metadata() const override;
         void set_metadata(const nlohmann::json& metadata) override;
@@ -335,8 +261,8 @@ namespace xt
 
         template <class CT = CTE>
         enable_assignable_t<CT> assign_impl(xarray<value_type>&& rhs);
-        
-        template <class CT = CTE>   
+
+        template <class CT = CTE>
         enable_not_assignable_t<CT> assign_impl(xarray<value_type>&& rhs);
 
         template <class CT = CTE>
@@ -364,6 +290,7 @@ namespace xt
         using value_type = typename std::decay_t<CTE>::value_type;
         using base_type = ztyped_array<value_type>;
         using shape_type = typename base_type::shape_type;
+        using slice_vector = typename base_type::slice_vector;
 
         template <class E>
         zscalar_wrapper(E&& e);
@@ -373,15 +300,17 @@ namespace xt
         virtual ~zscalar_wrapper() = default;
 
         bool is_array() const override;
+        bool is_chunked() const override;
 
         xarray<value_type>& get_array() override;
         const xarray<value_type>& get_array() const override;
+        xarray<value_type> get_chunk(const slice_vector& slices) const override;
 
         void assign(xarray<value_type>&& rhs) override;
 
         self_type* clone() const override;
 
-        zarray_impl* strided_view(xstrided_slice_vector& slices) override;
+        zarray_impl* strided_view(slice_vector& slices) override;
 
         const nlohmann::json& get_metadata() const override;
         void set_metadata(const nlohmann::json& metadata) override;
@@ -394,7 +323,7 @@ namespace xt
     private:
 
         zscalar_wrapper(const zscalar_wrapper&) = default;
-        
+
         CTE m_expression;
         xarray<value_type> m_array;
         nlohmann::json m_metadata;
@@ -413,6 +342,7 @@ namespace xt
         using value_type = typename std::decay_t<CTE>::value_type;
         using base_type = ztyped_array<value_type>;
         using shape_type = typename base_type::shape_type;
+        using slice_vector = typename base_type::slice_vector;
 
         template <class E>
         zarray_wrapper(E&& e);
@@ -420,15 +350,17 @@ namespace xt
         virtual ~zarray_wrapper() = default;
 
         bool is_array() const override;
+        bool is_chunked() const override;
 
         xarray<value_type>& get_array() override;
         const xarray<value_type>& get_array() const override;
+        xarray<value_type> get_chunk(const slice_vector& slices) const override;
 
         void assign(xarray<value_type>&& rhs) override;
 
         self_type* clone() const override;
 
-        zarray_impl* strided_view(xstrided_slice_vector& slices) override;
+        zarray_impl* strided_view(slice_vector& slices) override;
 
         const nlohmann::json& get_metadata() const override;
         void set_metadata(const nlohmann::json& metadata) override;
@@ -464,18 +396,33 @@ namespace xt
 
         virtual ~zchunked_array() = default;
         virtual const shape_type& chunk_shape() const = 0;
+        virtual size_t grid_size() const = 0;
+        virtual xstrided_slice_vector get_slice_vector(size_t chunk_index) const = 0;
+    };
+
+    template <class T>
+    class ztyped_chunked_array : public ztyped_array<T>,
+                                 public zchunked_array
+    {
+    public:
+
+        using value_type = T;
+
+        virtual ~ztyped_chunked_array() = default;
+
+        virtual void assign_chunk(xarray<value_type>&& rhs, size_t chunk_index) = 0;
     };
 
     template <class CTE>
-    class zchunked_wrapper : public ztyped_array<typename std::decay_t<CTE>::value_type>,
-                             public zchunked_array
+    class zchunked_wrapper : public ztyped_chunked_array<typename std::decay_t<CTE>::value_type>
     {
     public:
 
         using self_type = zchunked_wrapper;
-        using value_type = typename std::decay_t<CTE>::value_type;
-        using base_type = ztyped_array<value_type>;
+        using base_type = ztyped_chunked_array<typename std::decay_t<CTE>::value_type>;
+        using value_type = typename base_type::value_type;
         using shape_type = zchunked_array::shape_type;
+        using slice_vector = typename base_type::slice_vector;
 
         template <class E>
         zchunked_wrapper(E&& e);
@@ -483,15 +430,18 @@ namespace xt
         virtual ~zchunked_wrapper() = default;
 
         bool is_array() const override;
+        bool is_chunked() const override;
 
         xarray<value_type>& get_array() override;
         const xarray<value_type>& get_array() const override;
+        xarray<value_type> get_chunk(const slice_vector& slices) const override;
 
         void assign(xarray<value_type>&& rhs) override;
+        void assign_chunk(xarray<value_type>&& rhs, size_t chunk_index) override;
 
         self_type* clone() const override;
 
-        zarray_impl* strided_view(xstrided_slice_vector& slices) override;
+        zarray_impl* strided_view(slice_vector& slices) override;
 
         const nlohmann::json& get_metadata() const override;
         void set_metadata(const nlohmann::json& metadata) override;
@@ -502,6 +452,8 @@ namespace xt
         bool broadcast_shape(shape_type& shape, bool reuse_cache = 0) const override;
 
         const shape_type& chunk_shape() const override;
+        size_t grid_size() const override;
+        xstrided_slice_vector get_slice_vector(size_t chunk_index) const override;
 
     private:
 
@@ -514,7 +466,13 @@ namespace xt
 
         template <class CT = CTE>
         detail::disable_const_t<CT> assign_impl(xarray<value_type>&& rhs);
-        
+
+        template <class CT = CTE>
+        detail::enable_const_t<CT> assign_chunk_impl(xarray<value_type>&& rhs, size_t chunk_index);
+
+        template <class CT = CTE>
+        detail::disable_const_t<CT> assign_chunk_impl(xarray<value_type>&& rhs, size_t chunk_index);
+
         CTE m_chunked_array;
         shape_type m_chunk_shape;
         mutable xarray<value_type> m_cache;
@@ -526,6 +484,96 @@ namespace xt
 
     };
 
+    /*****************
+     * set_data_type *
+     *****************/
+
+    namespace detail
+    {
+        inline const std::string endianness_string()
+        {
+            static std::string endianness = (xtl::endianness() == xtl::endian::little_endian) ? "<" : ">";
+            return endianness;
+        }
+
+        template <class T>
+        inline void set_data_type(nlohmann::json& metadata)
+        {
+        }
+
+        template <>
+        inline void set_data_type<bool>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = "bool";
+        }
+
+        template <>
+        inline void set_data_type<uint8_t>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = "u1";
+        }
+
+        template <>
+        inline void set_data_type<int8_t>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = "i1";
+        }
+
+        template <>
+        inline void set_data_type<int16_t>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = endianness_string() + "i2";
+        }
+
+        template <>
+        inline void set_data_type<uint16_t>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = endianness_string() + "u2";
+        }
+
+        template <>
+        inline void set_data_type<int32_t>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = endianness_string() + "i4";
+        }
+
+        template <>
+        inline void set_data_type<uint32_t>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = endianness_string() + "u4";
+        }
+
+        template <>
+        inline void set_data_type<int64_t>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = endianness_string() + "i8";
+        }
+
+        template <>
+        inline void set_data_type<uint64_t>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = endianness_string() + "u8";
+        }
+
+        template <>
+        inline void set_data_type<xtl::half_float>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = endianness_string() + "f2";
+        }
+
+        template <>
+        inline void set_data_type<float>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = endianness_string() + "f4";
+        }
+
+        template <>
+        inline void set_data_type<double>(nlohmann::json& metadata)
+        {
+            metadata["data_type"] = endianness_string() + "f8";
+        }
+    }
+
     /***********************
      * zexpression_wrapper *
      ***********************/
@@ -538,7 +586,7 @@ namespace xt
         , m_cache()
         , m_cache_initialized(false)
     {
-        set_data_type<value_type>(m_metadata);
+        detail::set_data_type<value_type>(m_metadata);
     }
 
     template <class CTE>
@@ -547,6 +595,12 @@ namespace xt
         return false;
     }
 
+    template <class CTE>
+    bool zexpression_wrapper<CTE>::is_chunked() const
+    {
+        return false;
+    }
+
     template <class CTE>
     auto zexpression_wrapper<CTE>::get_array() -> xarray<value_type>&
     {
@@ -561,12 +615,18 @@ namespace xt
         return m_cache;
     }
 
+    template <class CTE>
+    auto zexpression_wrapper<CTE>::get_chunk(const slice_vector& slices) const -> xarray<value_type>
+    {
+        return xt::strided_view(m_expression, slices);
+    }
+
     template <class CTE>
     void zexpression_wrapper<CTE>::assign(xarray<value_type>&& rhs)
     {
         assign_impl(std::move(rhs));
     }
-    
+
     template <class CTE>
     auto zexpression_wrapper<CTE>::clone() const -> self_type*
     {
@@ -574,7 +634,7 @@ namespace xt
     }
 
     template <class CTE>
-    zarray_impl* zexpression_wrapper<CTE>::strided_view(xstrided_slice_vector& slices)
+    zarray_impl* zexpression_wrapper<CTE>::strided_view(slice_vector& slices)
     {
         return strided_view_impl(slices, detail::is_xstrided_view<CTE>());
     }
@@ -645,7 +705,7 @@ namespace xt
             m_cache_initialized = true;
         }
     }
-    
+
     template <class CTE>
     template <class CT>
     inline auto zexpression_wrapper<CTE>::assign_impl(xarray<value_type>&& rhs) -> enable_assignable_t<CT>
@@ -654,14 +714,14 @@ namespace xt
         // m_expression is not involved in rhs.
         xt::noalias(m_expression) = rhs;
     }
-    
+
     template <class CTE>
-    template <class CT>   
+    template <class CT>
     inline auto zexpression_wrapper<CTE>::assign_impl(xarray<value_type>&&) -> enable_not_assignable_t<CT>
     {
         throw std::runtime_error("unevaluated expression is not assignable");
     }
-    
+
     template <class CTE>
     template <class CT>
     inline auto zexpression_wrapper<CTE>::resize_impl() -> enable_assignable_t<CT>
@@ -675,7 +735,7 @@ namespace xt
     {
         throw std::runtime_error("cannot resize not assignable expression wrapper");
     }
-    
+
     /*******************
      * zscalar_wrapper *
      *******************/
@@ -687,7 +747,7 @@ namespace xt
         , m_expression(std::forward<E>(e))
         , m_array(m_expression())
     {
-        set_data_type<value_type>(m_metadata);
+        detail::set_data_type<value_type>(m_metadata);
     }
 
     template <class CTE>
@@ -696,6 +756,12 @@ namespace xt
         return false;
     }
 
+    template <class CTE>
+    bool zscalar_wrapper<CTE>::is_chunked() const
+    {
+        return false;
+    }
+
     template <class CTE>
     auto zscalar_wrapper<CTE>::get_array() -> xarray<value_type>&
     {
@@ -708,6 +774,12 @@ namespace xt
         return m_array;
     }
 
+    template <class CTE>
+    auto zscalar_wrapper<CTE>::get_chunk(const slice_vector&) const -> xarray<value_type>
+    {
+        return m_array;
+    }
+
     template <class CTE>
     void zscalar_wrapper<CTE>::assign(xarray<value_type>&&)
     {
@@ -721,7 +793,7 @@ namespace xt
     }
 
     template <class CTE>
-    zarray_impl* zscalar_wrapper<CTE>::strided_view(xstrided_slice_vector& slices)
+    zarray_impl* zscalar_wrapper<CTE>::strided_view(slice_vector& slices)
     {
         auto e = xt::strided_view(m_array, slices);
         return detail::build_zarray(std::move(e));
@@ -768,7 +840,7 @@ namespace xt
     {
         return m_array.broadcast_shape(shape, reuse_cache);
     }
-    
+
     /******************
      * zarray_wrapper *
      ******************/
@@ -808,7 +880,7 @@ namespace xt
         : base_type()
         , m_array(std::forward<E>(e))
     {
-        set_data_type<value_type>(m_metadata);
+        detail::set_data_type<value_type>(m_metadata);
     }
 
     template <class CTE>
@@ -817,6 +889,12 @@ namespace xt
         return true;
     }
 
+    template <class CTE>
+    bool zarray_wrapper<CTE>::is_chunked() const
+    {
+        return false;
+    }
+
     template <class CTE>
     auto zarray_wrapper<CTE>::get_array() -> xarray<value_type>&
     {
@@ -829,6 +907,12 @@ namespace xt
         return m_array;
     }
 
+    template <class CTE>
+    auto zarray_wrapper<CTE>::get_chunk(const slice_vector& slices) const -> xarray<value_type>
+    {
+        return xt::strided_view(m_array, slices);
+    }
+
     template <class CTE>
     void zarray_wrapper<CTE>::assign(xarray<value_type>&& rhs)
     {
@@ -842,7 +926,7 @@ namespace xt
     }
 
     template <class CTE>
-    zarray_impl* zarray_wrapper<CTE>::strided_view(xstrided_slice_vector& slices)
+    zarray_impl* zarray_wrapper<CTE>::strided_view(slice_vector& slices)
     {
         auto e = xt::strided_view(m_array, slices);
         return detail::build_zarray(std::move(e));
@@ -889,7 +973,7 @@ namespace xt
     {
         return m_array.broadcast_shape(shape, reuse_cache);
     }
-    
+
     template <class CTE>
     template <class CT>
     inline detail::enable_const_t<CT> zarray_wrapper<CTE>::assign_impl(xarray<value_type>&&)
@@ -903,7 +987,7 @@ namespace xt
     {
         m_array = std::move(rhs);
     }
-   
+
     /********************
      * zchunked_wrapper *
      ********************/
@@ -921,7 +1005,7 @@ namespace xt
         std::copy(m_chunked_array.chunk_shape().begin(),
                   m_chunked_array.chunk_shape().end(),
                   m_chunk_shape.begin());
-        set_data_type<value_type>(m_metadata);
+        detail::set_data_type<value_type>(m_metadata);
     }
 
     template <class CTE>
@@ -930,6 +1014,12 @@ namespace xt
         return false;
     }
 
+    template <class CTE>
+    bool zchunked_wrapper<CTE>::is_chunked() const
+    {
+        return true;
+    }
+
     template <class CTE>
     auto zchunked_wrapper<CTE>::get_array() -> xarray<value_type>&
     {
@@ -944,12 +1034,25 @@ namespace xt
         return m_cache;
     }
 
+    template <class CTE>
+    auto zchunked_wrapper<CTE>::get_chunk(const slice_vector& slices) const -> xarray<value_type>
+    {
+        compute_cache();
+        return xt::strided_view(m_cache, slices);
+    }
+
     template <class CTE>
     void zchunked_wrapper<CTE>::assign(xarray<value_type>&& rhs)
     {
         assign_impl(std::move(rhs));
     }
 
+    template <class CTE>
+    void zchunked_wrapper<CTE>::assign_chunk(xarray<value_type>&& rhs, size_t chunk_index)
+    {
+        assign_chunk_impl(std::move(rhs), chunk_index);
+    }
+
     template <class CTE>
     auto zchunked_wrapper<CTE>::clone() const -> self_type*
     {
@@ -957,7 +1060,7 @@ namespace xt
     }
 
     template <class CTE>
-    zarray_impl* zchunked_wrapper<CTE>::strided_view(xstrided_slice_vector& slices)
+    zarray_impl* zchunked_wrapper<CTE>::strided_view(slice_vector& slices)
     {
         auto e = xt::strided_view(m_chunked_array, slices);
         return detail::build_zarray(std::move(e));
@@ -1011,6 +1114,22 @@ namespace xt
         return m_chunked_array.broadcast_shape(shape, reuse_cache);
     }
 
+    template <class CTE>
+    size_t zchunked_wrapper<CTE>::grid_size() const
+    {
+        return m_chunked_array.grid_size();
+    }
+
+    template <class CTE>
+    xstrided_slice_vector zchunked_wrapper<CTE>::get_slice_vector(size_t chunk_index) const
+    {
+        // TODO: this is awful and should be replaced with a dynamic iterator wrapping
+        // the chunk iterator of the underlying chunked array.
+        auto it = m_chunked_array.chunk_begin();
+        std::advance(it, chunk_index);
+        return it.get_slice_vector();
+    }
+
     template <class CTE>
     auto zchunked_wrapper<CTE>::chunk_shape() const -> const shape_type&
     {
@@ -1022,7 +1141,8 @@ namespace xt
     {
         if (!m_cache_initialized)
         {
-            m_cache = m_chunked_array;
+            m_cache.resize(m_chunked_array.shape());
+            as_chunked(m_cache,  m_chunked_array.chunk_shape()) = m_chunked_array;
             m_cache_initialized = true;
         }
     }
@@ -1040,6 +1160,37 @@ namespace xt
     {
         xt::noalias(m_chunked_array) = rhs;
     }
+
+    template <class CTE>
+    template <class CT>
+    inline detail::enable_const_t<CT> zchunked_wrapper<CTE>::assign_chunk_impl(xarray<value_type>&&, size_t)
+    {
+        throw std::runtime_error("const array is not assignable");
+    }
+
+    template <class S>
+    void print_shape(const S& shape)
+    {
+        for (auto s: shape) { std::cout << s << ", "; }
+        std::cout << std::endl;
+    }
+
+    template <class CTE>
+    template <class CT>
+    inline detail::disable_const_t<CT> zchunked_wrapper<CTE>::assign_chunk_impl(xarray<value_type>&& rhs, size_t chunk_index)
+    {
+        auto it = m_chunked_array.chunk_begin();
+        std::advance(it, chunk_index);
+        const auto& chunk_shape = m_chunked_array.chunk_shape();
+        if (rhs.shape() != chunk_shape)
+        {
+            xt::noalias(xt::strided_view(*it, it.get_chunk_slice_vector())) = rhs;
+        }
+        else
+        {
+            xt::noalias(*it) = rhs;
+        }
+    }
 }
 
 #endif
